@@ -1,5 +1,7 @@
 import re
-from django.template import Library, Node, Variable, TemplateSyntaxError
+import types
+from copy import copy
+from django.template import Library, Node, TemplateSyntaxError
 register = Library()
 
 
@@ -12,25 +14,26 @@ def silence_without_field(fn):
 
 
 def _process_field_attributes(field, attr, process):
-
     # split attribute name and value from 'attr:value' string
-    params = attr.split(':', 1)
-    attribute = params[0]
-    value = params[1] if len(params) == 2 else ''
-
+    # params = attr.split(':', 1)
+    # attribute = params[0]
+    params = re.split(r'(?<!:):(?!:)', attr, 1)
+    # attribute = params[0]
+    attribute = params[0].replace('::', ':')
+    value = params[1] if len(params) == 2 else True
+    field = copy(field)
     # decorate field.as_widget method with updated attributes
     old_as_widget = field.as_widget
-
     def as_widget(self, widget=None, attrs=None, only_initial=False):
         attrs = attrs or {}
         process(widget or self.field.widget, attrs, attribute, value)
-        return old_as_widget(widget, attrs, only_initial)
-
-    bound_method = type(old_as_widget)
-    try:
-        field.as_widget = bound_method(as_widget, field, field.__class__)
-    except TypeError:  # python 3
-        field.as_widget = bound_method(as_widget, field)
+        if attribute == "type":  # change the Input type
+            self.field.widget.input_type = value
+            del attrs["type"]
+        html = old_as_widget(widget, attrs, only_initial)
+        self.as_widget = old_as_widget
+        return html
+    field.as_widget = types.MethodType(as_widget, field)
     return field
 
 
@@ -40,7 +43,6 @@ def set_attr(field, attr):
 
     def process(widget, attrs, attribute, value):
         attrs[attribute] = value
-
     return _process_field_attributes(field, attr, process)
 
 
@@ -71,6 +73,12 @@ def add_class(field, css_class):
     return append_attr(field, 'class:' + css_class)
 
 
+@register.filter("add_label_class")
+@silence_without_field
+def add_label_class(field, css_class):
+    return field.label_tag(attrs={'class': css_class})
+
+
 @register.filter("add_error_class")
 @silence_without_field
 def add_error_class(field, css_class):
@@ -113,7 +121,7 @@ def widget_type(field):
 
 ATTRIBUTE_RE = re.compile(r"""
     (?P<attr>
-        [\w_-]+
+        [@\w:_-]+
     )
     (?P<sign>
         \+?=
@@ -125,6 +133,7 @@ ATTRIBUTE_RE = re.compile(r"""
     )
 """, re.VERBOSE | re.UNICODE)
 
+
 @register.tag
 def render_field(parser, token):
     """
@@ -153,7 +162,8 @@ def render_field(parser, token):
         if not match:
             raise TemplateSyntaxError(error_msg + ": %s" % pair)
         dct = match.groupdict()
-        attr, sign, value = dct['attr'], dct['sign'], parser.compile_filter(dct['value'])
+        attr, sign, value = \
+            dct['attr'], dct['sign'], parser.compile_filter(dct['value'])
         if sign == "=":
             set_attrs.append((attr, value))
         else:
@@ -171,15 +181,22 @@ class FieldAttributeNode(Node):
     def render(self, context):
         bounded_field = self.field.resolve(context)
         field = getattr(bounded_field, 'field', None)
-        if (getattr(bounded_field, 'errors', None) and
-            'WIDGET_ERROR_CLASS' in context):
+        if (
+            getattr(bounded_field, 'errors', None) and
+            'WIDGET_ERROR_CLASS' in context
+        ):
             bounded_field = append_attr(bounded_field, 'class:%s' %
                                         context['WIDGET_ERROR_CLASS'])
         if field and field.required and 'WIDGET_REQUIRED_CLASS' in context:
             bounded_field = append_attr(bounded_field, 'class:%s' %
                                         context['WIDGET_REQUIRED_CLASS'])
         for k, v in self.set_attrs:
-            bounded_field = set_attr(bounded_field, '%s:%s' % (k,v.resolve(context)))
+            if k == 'type':
+                bounded_field.field.widget.input_type = v.resolve(context)
+            else:
+                bounded_field = \
+                    set_attr(bounded_field, '%s:%s' % (k, v.resolve(context)))
         for k, v in self.append_attrs:
-            bounded_field = append_attr(bounded_field, '%s:%s' % (k,v.resolve(context)))
-        return bounded_field
+            bounded_field = \
+                append_attr(bounded_field, '%s:%s' % (k, v.resolve(context)))
+        return str(bounded_field)
