@@ -1,15 +1,20 @@
 import string
-try:
-    from django.utils.unittest import expectedFailure
-except ImportError:
-    def expectedFailure(func):
-        return lambda *args, **kwargs: None
 
-from django.test import TestCase
-from django.forms import Form, CharField, TextInput
+try:
+    from unittest import TestCase, skipIf
+except ImportError:
+    from unittest2 import TestCase, skipIf
+
+from django import VERSION
 from django import forms
+from django.forms import Form, CharField, TextInput
 from django.template import Template, Context
-from django.forms.extras.widgets import SelectDateWidget
+
+try:
+    from django.forms import SelectDateWidget  # django >= 2.0
+except ImportError:
+    from django.forms.extras.widgets import SelectDateWidget  # django < 2.0
+
 
 # ==============================
 #       Testing helpers
@@ -23,10 +28,10 @@ class MyForm(Form):
     """
     simple = CharField()
     with_attrs = CharField(widget=TextInput(attrs={
-                    'foo': 'baz',
-                    'egg': 'spam'
-                 }))
-    with_cls = CharField(widget=TextInput(attrs={'class':'class0'}))
+        'foo': 'baz',
+        'egg': 'spam'
+    }))
+    with_cls = CharField(widget=TextInput(attrs={'class': 'class0'}))
     date = forms.DateField(widget=SelectDateWidget(attrs={'egg': 'spam'}))
 
 
@@ -114,6 +119,16 @@ class SimpleAttrTest(TestCase):
         res = render_field('simple', 'set_data', 'key:value')
         assertIn('data-key="value"', res)
 
+    def test_replace_type(self):
+        res = render_field('simple', 'attr', 'type:date')
+        self.assertTrue(res.count("type=") == 1, (res, res.count("type=")))
+        assertIn('type="date"', res)
+
+    def test_replace_hidden(self):
+        res = render_field('simple', 'attr', 'type:hidden')
+        self.assertTrue(res.count("type=") == 1, (res, res.count("type=")))
+        assertIn('type="hidden"', res)
+
 
 class ErrorsTest(TestCase):
 
@@ -156,8 +171,8 @@ class CustomizedWidgetTest(TestCase):
         assertNotIn('foo="baz"', res)
         assertIn('egg="spam"', res)
 
-    # see https://code.djangoproject.com/ticket/16754
-    @expectedFailure
+    # XXX can be dropped once 1.8 is not supported
+    @skipIf(VERSION < (1, 11, 0, 'final', 0), 'see https://code.djangoproject.com/ticket/16754')
     def test_selectdatewidget(self):
         res = render_field('date', 'attr', 'foo:bar')
         assertIn('egg="spam"', res)
@@ -241,8 +256,8 @@ class RenderFieldTagCustomizedWidgetTest(TestCase):
         assertNotIn('foo="baz"', res)
         assertIn('egg="spam"', res)
 
-    # see https://code.djangoproject.com/ticket/16754
-    @expectedFailure
+    # XXX can be dropped once 1.8 is not supported
+    @skipIf(VERSION < (1, 11, 0, 'final', 0), 'see https://code.djangoproject.com/ticket/16754')
     def test_selectdatewidget(self):
         res = render_field_from_tag('date', 'foo="bar"')
         assertIn('egg="spam"', res)
@@ -286,22 +301,22 @@ class RenderFieldWidgetClassesTest(TestCase):
     def test_use_widget_required_class(self):
         res = render_form('{% render_field form.simple %}',
                           WIDGET_REQUIRED_CLASS='required_class')
-        self.assertIn('class="required_class"', res)
+        assertIn('class="required_class"', res)
 
     def test_use_widget_error_class(self):
         res = render_form('{% render_field form.simple %}', form=MyForm({}),
                           WIDGET_ERROR_CLASS='error_class')
-        self.assertIn('class="error_class"', res)
+        assertIn('class="error_class"', res)
 
     def test_use_widget_error_class_with_other_classes(self):
         res = render_form('{% render_field form.simple class="blue" %}',
                           form=MyForm({}), WIDGET_ERROR_CLASS='error_class')
-        self.assertIn('class="blue error_class"', res)
+        assertIn('class="blue error_class"', res)
 
     def test_use_widget_required_class_with_other_classes(self):
         res = render_form('{% render_field form.simple class="blue" %}',
                           form=MyForm({}), WIDGET_REQUIRED_CLASS='required_class')
-        self.assertIn('class="blue required_class"', res)
+        assertIn('class="blue required_class"', res)
 
 
 class RenderFieldTagFieldReuseTest(TestCase):
@@ -319,6 +334,48 @@ class RenderFieldTagFieldReuseTest(TestCase):
         self.assertEqual(res.count("class0"), 3)
         self.assertEqual(res.count("bar"), 1)
 
+    def test_field_double_rendering_id(self):
+        res = render_form(
+            '{{ form.simple }}'
+            '{% render_field form.simple id="id_1" %}'
+            '{% render_field form.simple id="id_2" %}'
+        )
+        self.assertEqual(res.count("id_1"), 1)
+        self.assertEqual(res.count("id_2"), 1)
+
+    def test_field_double_rendering_id_name(self):
+        res = render_form(
+            '{{ form.simple }}'
+            '{% render_field form.simple id="id_1" name="n_1" %}'
+            '{% render_field form.simple id="id_2" name="n_2" %}'
+        )
+        self.assertEqual(res.count("id_1"), 1)
+        self.assertEqual(res.count("id_2"), 1)
+        self.assertEqual(res.count("n_1"), 1)
+        self.assertEqual(res.count("n_2"), 1)
+
+    def test_field_double_rendering_id_class(self):
+        res = render_form(
+            '{{ form.simple }}'
+            '{% render_field form.simple id="id_1" class="c_1" %}'
+            '{% render_field form.simple id="id_2" class="c_2" %}'
+        )
+        self.assertEqual(res.count("id_1"), 1)
+        self.assertEqual(res.count("id_2"), 1)
+        self.assertEqual(res.count("c_1"), 1)
+        self.assertEqual(res.count("c_2"), 1)
+
+    def test_field_double_rendering_name_class(self):
+        res = render_form(
+            '{{ form.simple }}'
+            '{% render_field form.simple name="n_1" class="c_1" %}'
+            '{% render_field form.simple name="n_2" class="c_2" %}'
+        )
+        self.assertEqual(res.count("n_1"), 1)
+        self.assertEqual(res.count("n_2"), 1)
+        self.assertEqual(res.count("c_1"), 1)
+        self.assertEqual(res.count("c_2"), 1)
+
 
 class RenderFieldTagUseTemplateVariableTest(TestCase):
     def test_use_template_variable_in_parametrs(self):
@@ -331,3 +388,37 @@ class RenderFieldFilter_field_type_widget_type_Test(TestCase):
     def test_field_type_widget_type_rendering_simple(self):
         res = render_form('<div class="{{ form.simple|field_type }} {{ form.simple|widget_type }} {{ form.simple.html_name }}">{{ form.simple }}</div>')
         assertIn('class="charfield textinput simple"', res)
+
+
+class RenderFieldTagNonValueAttribute(TestCase):
+    def test_field_non_value(self):
+        res = render_form('{{ form.simple|attr:"foo" }}')
+        assertIn('foo', res)
+        assertNotIn('foo=', res)
+
+    def test_field_empty_value(self):
+        res = render_form('{{ form.simple|attr:"foo:" }}')
+        assertIn('foo=""', res)
+
+    def test_field_other_value(self):
+        res = render_form('{{ form.simple|attr:"foo:bar" }}')
+        assertIn('foo="bar"', res)
+
+    def test_field_double_colon(self):
+        res = render_form('{{ form.simple|attr:"v-bind::class:value" }}')
+        assertIn('v-bind:class="value"', res)
+
+    def test_field_double_colon_morethanone(self):
+        res = render_form('{{ form.simple|attr:"v-bind::class:{active:True}" }}')
+        assertIn('v-bind:class="{active:True}"', res)
+    def test_field_arroba(self):
+        res = render_form('{{ form.simple|attr:"@click:onClick" }}')
+        assertIn('@click="onClick"', res)
+    def test_field_arroba_dot(self):
+        res = render_form('{{ form.simple|attr:"@click.prevent:onClick" }}')
+        assertIn('@click.prevent="onClick"', res)
+
+    def test_field_double_colon_missing(self):
+        res = render_form('{{ form.simple|attr:"::class:{active:True}" }}')
+        assertIn(':class="{active:True}"', res)
+
